<?php

namespace Bixo\Accounts\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Accounts\Http\Requests\AccountsRequest;
use Bixo\Accounts\Interfaces\AccountsRepositoryInterface;
use Bixo\Accounts\Models\Accounts;
use Bixo\Accounts\Forms\Accounts as Form;

/**
 * APIController  class for accounts.
 */
class AccountsAPIController extends BaseController
{

    /**
     * Initialize accounts resource controller.
     *
     * @param type AccountsRepositoryInterface $accounts
     *
     * @return null
     */
    public function __construct(AccountsRepositoryInterface $accounts)
    {
        parent::__construct();
        $this->repository = $accounts;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Accounts\Repositories\Criteria\AccountsResourceCriteria::class);
    }

    /**
     * Display a list of accounts.
     *
     * @return Response
     */
    public function index(AccountsRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Accounts\Repositories\Presenter\AccountsPresenter::class)
            ->paginate();
    }

    /**
     * Display accounts.
     *
     * @param Request $request
     * @param Model   $accounts
     *
     * @return Response
     */
    public function show(AccountsRequest $request, Accounts $accounts)
    {
        return $accounts->setPresenter(\Bixo\Accounts\Repositories\Presenter\AccountsListPresenter::class);
        ;
    }

    /**
     * Create new accounts.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AccountsRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('accounts::accounts.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('accounts/accounts/' . $accounts->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('accounts/accounts');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the accounts.
     *
     * @param Request $request
     * @param Model   $accounts
     *
     * @return Response
     */
    public function update(AccountsRequest $request, Accounts $accounts)
    {
        try {
            $data = $request->all();

            $accounts->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('accounts::accounts.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('accounts/accounts/' . $accounts->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('accounts/accounts/' . $accounts->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the accounts.
     *
     * @param Model   $accounts
     *
     * @return Response
     */
    public function destroy(AccountsRequest $request, Accounts $accounts)
    {
        try {
            $accounts->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('accounts::accounts.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('accounts/accounts/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('accounts/accounts/' . $accounts->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
